@@ -4,6 +4,7 @@ use std::sync::atomic::{AtomicUsize, Ordering};
 use pyo3::prelude::*;
 use pyo3::types::{PyBytes, PyDict, PyTuple, PyType};
 use pyo3::{PyTraverseError, PyVisit};
+use type_serializers::any::AnySerializer;
 
 use crate::definitions::{Definitions, DefinitionsBuilder};
 use crate::py_gc::PyGcTraverse;
@@ -12,9 +13,9 @@ pub(crate) use config::BytesMode;
 use config::SerializationConfig;
 pub use errors::{PydanticSerializationError, PydanticSerializationUnexpectedValue};
 use extra::{CollectWarnings, SerRecursionState, WarningsMode};
-pub(crate) use extra::{DuckTypingSerMode, Extra, SerMode, SerializationState};
+pub(crate) use extra::{Extra, SerMode, SerializationState};
+use shared::to_json_bytes;
 pub use shared::CombinedSerializer;
-use shared::{to_json_bytes, TypeSerializer};
 
 mod computed_fields;
 mod config;
@@ -63,7 +64,7 @@ impl SchemaSerializer {
         rec_guard: &'a SerRecursionState,
         serialize_unknown: bool,
         fallback: Option<&'a Bound<'a, PyAny>>,
-        duck_typing_ser_mode: DuckTypingSerMode,
+        serialize_as_any: bool,
         context: Option<&'a Bound<'a, PyAny>>,
     ) -> Extra<'b> {
         Extra::new(
@@ -79,7 +80,7 @@ impl SchemaSerializer {
             rec_guard,
             serialize_unknown,
             fallback,
-            duck_typing_ser_mode,
+            serialize_as_any,
             context,
         )
     }
@@ -133,7 +134,6 @@ impl SchemaSerializer {
         };
         let warnings = CollectWarnings::new(warnings_mode);
         let rec_guard = SerRecursionState::default();
-        let duck_typing_ser_mode = DuckTypingSerMode::from_bool(serialize_as_any);
         let extra = self.build_extra(
             py,
             &mode,
@@ -146,7 +146,7 @@ impl SchemaSerializer {
             &rec_guard,
             false,
             fallback,
-            duck_typing_ser_mode,
+            serialize_as_any,
             context,
         );
         let v = self.serializer.to_python(value, include, exclude, &extra)?;
@@ -155,7 +155,7 @@ impl SchemaSerializer {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (value, *, indent = None, include = None, exclude = None, by_alias = None,
+    #[pyo3(signature = (value, *, indent = None, ensure_ascii = false, include = None, exclude = None, by_alias = None,
         exclude_unset = false, exclude_defaults = false, exclude_none = false, round_trip = false, warnings = WarningsArg::Bool(true),
         fallback = None, serialize_as_any = false, context = None))]
     pub fn to_json(
@@ -163,6 +163,7 @@ impl SchemaSerializer {
         py: Python,
         value: &Bound<'_, PyAny>,
         indent: Option<usize>,
+        ensure_ascii: Option<bool>,
         include: Option<&Bound<'_, PyAny>>,
         exclude: Option<&Bound<'_, PyAny>>,
         by_alias: Option<bool>,
@@ -181,7 +182,6 @@ impl SchemaSerializer {
         };
         let warnings = CollectWarnings::new(warnings_mode);
         let rec_guard = SerRecursionState::default();
-        let duck_typing_ser_mode = DuckTypingSerMode::from_bool(serialize_as_any);
         let extra = self.build_extra(
             py,
             &SerMode::Json,
@@ -194,7 +194,7 @@ impl SchemaSerializer {
             &rec_guard,
             false,
             fallback,
-            duck_typing_ser_mode,
+            serialize_as_any,
             context,
         );
         let bytes = to_json_bytes(
@@ -204,6 +204,7 @@ impl SchemaSerializer {
             exclude,
             &extra,
             indent,
+            ensure_ascii.unwrap_or(false),
             self.expected_json_size.load(Ordering::Relaxed),
         )?;
 
@@ -239,7 +240,7 @@ impl SchemaSerializer {
 
 #[allow(clippy::too_many_arguments)]
 #[pyfunction]
-#[pyo3(signature = (value, *, indent = None, include = None, exclude = None, by_alias = true,
+#[pyo3(signature = (value, *, indent = None, ensure_ascii = false, include = None, exclude = None, by_alias = true,
     exclude_none = false, round_trip = false, timedelta_mode = "iso8601", bytes_mode = "utf8",
     inf_nan_mode = "constants", serialize_unknown = false, fallback = None, serialize_as_any = false,
     context = None))]
@@ -247,6 +248,7 @@ pub fn to_json(
     py: Python,
     value: &Bound<'_, PyAny>,
     indent: Option<usize>,
+    ensure_ascii: Option<bool>,
     include: Option<&Bound<'_, PyAny>>,
     exclude: Option<&Bound<'_, PyAny>>,
     by_alias: bool,
@@ -261,7 +263,6 @@ pub fn to_json(
     context: Option<&Bound<'_, PyAny>>,
 ) -> PyResult<PyObject> {
     let state = SerializationState::new(timedelta_mode, bytes_mode, inf_nan_mode)?;
-    let duck_typing_ser_mode = DuckTypingSerMode::from_bool(serialize_as_any);
     let extra = state.extra(
         py,
         &SerMode::Json,
@@ -270,11 +271,19 @@ pub fn to_json(
         round_trip,
         serialize_unknown,
         fallback,
-        duck_typing_ser_mode,
+        serialize_as_any,
         context,
     );
-    let serializer = type_serializers::any::AnySerializer.into();
-    let bytes = to_json_bytes(value, &serializer, include, exclude, &extra, indent, 1024)?;
+    let bytes = to_json_bytes(
+        value,
+        AnySerializer::get(),
+        include,
+        exclude,
+        &extra,
+        indent,
+        ensure_ascii.unwrap_or(false),
+        1024,
+    )?;
     state.final_check(py)?;
     let py_bytes = PyBytes::new(py, &bytes);
     Ok(py_bytes.into())
@@ -302,7 +311,6 @@ pub fn to_jsonable_python(
     context: Option<&Bound<'_, PyAny>>,
 ) -> PyResult<PyObject> {
     let state = SerializationState::new(timedelta_mode, bytes_mode, inf_nan_mode)?;
-    let duck_typing_ser_mode = DuckTypingSerMode::from_bool(serialize_as_any);
     let extra = state.extra(
         py,
         &SerMode::Json,
@@ -311,7 +319,7 @@ pub fn to_jsonable_python(
         round_trip,
         serialize_unknown,
         fallback,
-        duck_typing_ser_mode,
+        serialize_as_any,
         context,
     );
     let v = infer::infer_to_python(value, include, exclude, &extra)?;
