@@ -214,11 +214,12 @@ impl TupleSerializer {
                 .chain(self.serializers[variadic_item_index + 1..].iter());
             use_serializers!(serializers_iter);
         } else if extra.check == SerCheck::Strict && n_items != self.serializers.len() {
-            return Err(PydanticSerializationUnexpectedValue::new_err(Some(format!(
+            return Err(PydanticSerializationUnexpectedValue::new_from_msg(Some(format!(
                 "Expected {} items, but got {}",
                 self.serializers.len(),
                 n_items
-            ))));
+            )))
+            .to_py_err());
         } else {
             use_serializers!(self.serializers.iter());
             let mut warned = false;
@@ -226,7 +227,9 @@ impl TupleSerializer {
                 if !warned {
                     extra
                         .warnings
-                        .custom_warning("Unexpected extra items present in tuple".to_string());
+                        .register_warning(PydanticSerializationUnexpectedValue::new_from_msg(Some(
+                            "Unexpected extra items present in tuple".to_string(),
+                        )));
                     warned = true;
                 }
                 let op_next = self
@@ -240,10 +243,10 @@ impl TupleSerializer {
                         serializer: &CombinedSerializer::Any(AnySerializer),
                     }) {
                         return Ok(Err(e));
-                    };
+                    }
                 }
             }
-        };
+        }
         Ok(Ok(()))
     }
 }
