@@ -1,9 +1,11 @@
 import dataclasses
-import re
 from datetime import date
+from enum import Enum
 from typing import Any
 
 import pytest
+from typing_extensions import get_args, get_type_hints  # noqa: UP035 (https://github.com/astral-sh/ruff/pull/18476)
+from typing_inspection.introspection import UNKNOWN, AnnotationSource, inspect_annotation
 
 from pydantic_core import SchemaError, SchemaSerializer, SchemaValidator, ValidationError, core_schema
 
@@ -26,6 +28,11 @@ class MyDataclass:
     y: str
 
 
+class MyEnum(int, Enum):
+    a = 1
+    b = 2
+
+
 def ids_function(val):
     if callable(val):
         return val.__name__
@@ -75,21 +82,16 @@ all_schema_functions = [
         {'type': 'timedelta', 'microseconds_precision': 'error'},
     ),
     (core_schema.literal_schema, args(['a', 'b']), {'type': 'literal', 'expected': ['a', 'b']}),
+    (
+        core_schema.enum_schema,
+        args(MyEnum, list(MyEnum.__members__.values())),
+        {'type': 'enum', 'cls': MyEnum, 'members': [MyEnum.a, MyEnum.b]},
+    ),
     (core_schema.is_instance_schema, args(int), {'type': 'is-instance', 'cls': int}),
     (core_schema.callable_schema, args(), {'type': 'callable'}),
     (core_schema.list_schema, args(), {'type': 'list'}),
     (core_schema.list_schema, args({'type': 'int'}), {'type': 'list', 'items_schema': {'type': 'int'}}),
-    (
-        core_schema.tuple_positional_schema,
-        args([{'type': 'int'}]),
-        {'type': 'tuple-positional', 'items_schema': [{'type': 'int'}]},
-    ),
-    (core_schema.tuple_positional_schema, args([]), {'type': 'tuple-positional', 'items_schema': []}),
-    (
-        core_schema.tuple_variable_schema,
-        args({'type': 'int'}),
-        {'type': 'tuple-variable', 'items_schema': {'type': 'int'}},
-    ),
+    (core_schema.tuple_schema, args([]), {'type': 'tuple', 'items_schema': []}),
     (
         core_schema.set_schema,
         args({'type': 'int'}, min_length=4),
@@ -224,6 +226,24 @@ all_schema_functions = [
             'serialization': {'type': 'format', 'formatting_string': 'd'},
         },
     ),
+    (
+        core_schema.arguments_v3_schema,
+        args(
+            [
+                core_schema.arguments_v3_parameter('foo', core_schema.int_schema()),
+                core_schema.arguments_v3_parameter('bar', core_schema.str_schema()),
+            ],
+            serialization=core_schema.format_ser_schema('d'),
+        ),
+        {
+            'type': 'arguments-v3',
+            'arguments_schema': [
+                {'name': 'foo', 'schema': {'type': 'int'}},
+                {'name': 'bar', 'schema': {'type': 'str'}},
+            ],
+            'serialization': {'type': 'format', 'formatting_string': 'd'},
+        },
+    ),
     (
         core_schema.call_schema,
         args(core_schema.arguments_schema([core_schema.arguments_parameter('foo', {'type': 'int'})]), val_function),
@@ -289,6 +309,8 @@ all_schema_functions = [
     (core_schema.uuid_schema, args(), {'type': 'uuid'}),
     (core_schema.decimal_schema, args(), {'type': 'decimal'}),
     (core_schema.decimal_schema, args(multiple_of=5, gt=1.2), {'type': 'decimal', 'multiple_of': 5, 'gt': 1.2}),
+    (core_schema.complex_schema, args(), {'type': 'complex'}),
+    (core_schema.invalid_schema, args(), {'type': 'invalid'}),
 ]
 
 
@@ -297,7 +319,7 @@ def test_schema_functions(function, args_kwargs, expected_schema):
     args, kwargs = args_kwargs
     schema = function(*args, **kwargs)
     assert schema == expected_schema
-    if schema.get('type') in {None, 'definition-ref', 'typed-dict-field', 'model-field'}:
+    if schema.get('type') in {None, 'definition-ref', 'typed-dict-field', 'model-field', 'invalid'}:
         return
 
     v = SchemaValidator(schema)
@@ -311,10 +333,14 @@ def test_schema_functions(function, args_kwargs, expected_schema):
 
 
 def test_all_schema_functions_used():
-    all_types = {
-        re.sub(r".+'(.+?)'.+", r'\1', s.__annotations__['type'].__forward_arg__)
-        for s in core_schema.CoreSchema.__args__
-    }
+    all_types: set[str] = set()
+    for schema_typeddict in core_schema.CoreSchema.__args__:
+        annotation = get_type_hints(schema_typeddict, include_extras=True)['type']
+        inspected_ann = inspect_annotation(annotation, annotation_source=AnnotationSource.TYPED_DICT)
+        annotation = inspected_ann.type
+        assert annotation is not UNKNOWN
+        all_types.add(get_args(annotation)[0])
+
     types_used = {args['type'] for _, _, args in all_schema_functions if 'type' in args}
 
     # isn't a CoreSchema type
@@ -352,3 +378,8 @@ def test_expected_serialization_types(return_schema):
             )
         )
     )
+
+
+def test_err_on_invalid() -> None:
+    with pytest.raises(SchemaError, match='Cannot construct schema with `InvalidSchema` member.'):
+        SchemaValidator(core_schema.invalid_schema())
