@@ -1,4 +1,5 @@
 import platform
+import sys
 import weakref
 
 import pytest
@@ -411,7 +412,7 @@ def test_model_custom_init_revalidate():
     assert calls == ["{'a': '1'}", "{'a': '1', 'x': 4}"]
 
 
-@pytest.mark.xfail(is_free_threaded, reason='GC leaks on free-threaded')
+@pytest.mark.xfail(is_free_threaded and sys.version_info < (3, 14), reason='GC leaks on free-threaded (<3.14)')
 @pytest.mark.xfail(
     condition=platform.python_implementation() == 'PyPy', reason='https://foss.heptapod.net/pypy/pypy/-/issues/3899'
 )
