@@ -627,16 +627,22 @@ def test_use_default_error() -> None:
     validator = SchemaValidator(
         core_schema.with_default_schema(core_schema.no_info_wrap_validator_function(val_func, core_schema.int_schema()))
     )
-    with pytest.raises(SchemaError, match='Uncaught UseDefault error, please check your usage of `default` validators'):
+    with pytest.raises(
+        SchemaError,
+        match='Uncaught `PydanticUseDefault` exception: the error was raised in a field validator and no default value is available for that field.',
+    ):
         validator.validate_python('')
 
     # same if there is no WithDefault validator
     validator = SchemaValidator(core_schema.no_info_wrap_validator_function(val_func, core_schema.int_schema()))
-    with pytest.raises(SchemaError, match='Uncaught UseDefault error, please check your usage of `default` validators'):
+    with pytest.raises(
+        SchemaError,
+        match='Uncaught `PydanticUseDefault` exception: the error was raised in a field validator and no default value is available for that field.',
+    ):
         validator.validate_python('')
 
 
-@pytest.mark.xfail(is_free_threaded, reason='GC leaks on free-threaded')
+@pytest.mark.xfail(is_free_threaded and sys.version_info < (3, 14), reason='GC leaks on free-threaded (<3.14)')
 @pytest.mark.xfail(
     condition=platform.python_implementation() == 'PyPy', reason='https://foss.heptapod.net/pypy/pypy/-/issues/3899'
 )
